////////////////////////////////////////////////////////////////////////////
//
//  COM_SELECT.PAS
//
//  Ein einfacher Dialog zur Auswahl der seriellen Schnittstelle (und Baudrate)
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit comselect;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,

  simple232, strfuncs;

type
  // implementattion der dialog klasse (als form)
  TfrmComSelect = class(TForm)
    cbComPort: TComboBox;
    cbBaudRate: TComboBox;
    lbComPort: TLabel;
    lbBaudRate: TLabel;
    btOK: TButton;
    btCancel: TButton;
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure SetupDialog (comport : integer; baudrate : integer);
    procedure GetComSettings (var comport : integer; var baudrate : integer);
  end;

  // implementation der dialog klasse (als komponente)
  TComSelectDialog = class (TComponent)
  private
    frm : TfrmComSelect;
  public
    constructor create (aowner : tcomponent); override;
    destructor Destroy; override;

    function Execute (var comport : integer; var baudrate : integer) : boolean;
  end;

// aufruf des dialogs (stand-alone)
function ComPortSelect (var comport : integer; var baudrate : integer) : boolean; forward;

var
  frmComSelect: TfrmComSelect;

implementation

{$R *.dfm}

procedure TfrmComSelect.FormCreate(Sender: TObject);
var x, y : ansistring;
    i : integer;
begin
  // com-port combobox lschen
  cbComPort.Items.Clear;
  // verfgbare com-ports ermitteln (string der durch komma separierte werte enthlt)
  x := GetAvailableComs;
  if x <> '' then
  begin
    // jedes komma ein com-port ...
    repeat
      y := getuntil (x, ',', x);
      if y <> '' then
      begin
        // zur combobox hinzufgen
        i := cbComPort.Items.Add('COM' + y);
        cbComPort.Items.Objects [i] := pointer (strtoint (y));
      end;
    until x = '';
  end;
  // baudrate std-mssig auf 38400 einstellen
  cbBaudRate.ItemIndex := 8;
end;

// dialog vorbesetzen
procedure TfrmComSelect.SetupDialog (comport : integer; baudrate : integer);
var i : integer;
begin
  // existiert vorgegebener com-port in liste ?
  i := cbComPort.Items.IndexOf (format ('COM%d', [comport]));
  // wenn ja, auswahlen, ansonsten ersten nehmen
  if i <> -1 then cbComPort.ItemIndex := i
             else cbComPort.ItemIndex := 0;
  // ist die baudrate vorgegeben ?
  if baudrate <> -1 then
  begin
    // danach suchen ...
    i := cbBaudRate.Items.IndexOf (format ('%d', [baudrate]));
    // und wenn existent, dann setzen (und baudraten-feld deaktivieren)
    if i <> -1 then
    begin
      cbBaudRate.ItemIndex := i;
      cbBaudRate.Enabled := false;
    end;
  end;
end;

// aktuelle einstellung des dialogs lesen
procedure TfrmComSelect.GetComSettings (var comport : integer; var baudrate : integer);
begin
  comport := 1; baudrate := 38400;
  if cbComPort.ItemIndex  <> -1 then comport := longint (cbcomport.Items.objects [cbcomport.ItemIndex]);
  if cbbaudrate.ItemIndex <> -1 then baudrate := strtoint (cbbaudrate.Items.strings [cbbaudrate.ItemIndex]);
end;




// kapselung des forumlars zu einem dialog
constructor TComSelectDialog.create (aowner : tcomponent);
begin
  inherited create (aowner);
  frm := TfrmComSelect.Create (self);
end;


destructor TComSelectDialog.Destroy;
begin
  frm.Destroy;
  inherited;
end;

// dialog aufsetzen, ausfhren und anschlieend auslesen
function TComSelectDialog.Execute (var comport : integer; var baudrate : integer) : boolean;
begin
  frm.SetupDialog(comport, baudrate);
  result := frm.ShowModal = mrOK;
  frm.GetComSettings(comport, baudrate);
end;


// kapselung des dialogs als stand-alone funktion
function ComPortSelect (var comport : integer; var baudrate : integer) : boolean;
var csd : TComSelectDialog;
begin
  csd := TComSelectDialog.create(NIL);
  result := csd.Execute(comport, baudrate);
  csd.Destroy;
end;


end.
