////////////////////////////////////////////////////////////////////////////
//
//  SIMPLE232.PAS
//
//  Einfache RS232 Klasse zum senden und empfangen. Weiterhin gibt es
//  ein paar ntzliche Funktionen (verfgnare Comports, usw).
//  Die Grundfunktionen werden im Pascal-Stil (klassenlos) gekapselt, und
//  von TSimple232 aufgerufen. TSimple232 stellt einen Polling-Mechanismus
//  zur Verfgung der bei eintreffen von Daten eine Callback aufruft.
//
//  Hinweis : Ich habe es mir einfach gemacht und alle Buffer als string
//            (ansistring) behandelt
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit simple232;

interface

uses windows, extctrls, dialogs, sysutils;


     // callback fr empfang
type TReceivedata = procedure (clSender : TObject; aucData : string) of object;

type TSimple232 = class
     private
       // timer fr polling
       clTimer1 : TTimer;

       // polling-routine
       procedure vPoll (clSender : TObject);
       // status (geffnet oder nicht)
       function F_bComOk : boolean;

     public
       // callback
       pfOnReceiveData : TReceivedata;

       constructor Create ;
       destructor  Destroy; override;

       // comport fnnen und schlieen
       function  OpenPort (ucComPort : byte; uiBaudRate : dword) : boolean;
       procedure ClosePort;

       // daten senden
       procedure vSendData (aucData : string);

       // status auslesen
       property bComOk : boolean read F_bComOk;
     end;



// com-port scan obergrenze
const MAX_COM_PORT_SCAN : integer = 10;

// alle verfgbaren comports (innerhalb der grenzen von 1 - MAX_COM_PORT_SCAN)
function  GetAvailableComs : string; forward;


implementation


//////////////////////////////////////////////////////////////////////////////
//
//
//  Kapselung der Schnittstellenfunktionen in "lose" Pascal Funktionen
//
//
//////////////////////////////////////////////////////////////////////////////



// globale com-port handle (es kann also nur 1 port gleichzeitig geffnet werden)
var tComPortHandle : THANDLE;


// initialisieren des handles
procedure InitCom;
begin
  tComPortHandle := INVALID_HANDLE_VALUE;
end;

// offnen eines com-ports (ohne baudrate zu setzen)
function OpenComPort (ucPort : byte) : boolean;
var aucComStr               : string [20];
    pucComStrPtr            : pointer;
    tTempHandle             : THANDLE;
begin
  result := false;
  // nur wenn noch kein com-port geffnet ist
  if tComPortHandle = INVALID_HANDLE_VALUE then
  begin
    // namen zusammenbauen, terminieren und pointer ermitteln
    aucComStr    := format ('COM%d', [ucPort]) + #0;
    pucComStrPtr := pointer (longint (@aucComStr) + 1);
    // letzenendes schnittstelle ffnen
    tTempHandle  := CreateFile (pucComStrPtr, GENERIC_READ or GENERIC_WRITE, 0,
                                NIL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

    // und auf fehler prfen und status zurckgeben                                
    if tTempHandle <> INVALID_HANDLE_VALUE then
      begin
        tComPortHandle := tTempHandle;
        result := true;
      end
    else
      begin
        tComPortHandle := INVALID_HANDLE_VALUE;
        result := false;
      end;
  end;
end;

// com-port schlieen
function CloseComPort : boolean;
begin
  if tComPortHandle <> INVALID_HANDLE_VALUE then
    begin
      result := CloseHandle (tComPortHandle);
      tComPortHandle := INVALID_HANDLE_VALUE;
    end
  else
    result := false;
end;

// verfgbare schnittstellen ermitteln und als string zurckliefern
function  GetAvailableComs : string;
var c : integer;
    y : string;
begin
  // alle schnittstellen abklappern
  for c := 1 to MAX_COM_PORT_SCAN do
  begin
    // test
    if OpenComPort (c) then
    begin
      // ok -> hinzufgen
      y := y + format ('%d,', [c]);
      CloseComPort;
    end
  end;
  // letztes komma abschneiden
  result := copy (y, 1, length (y) - 1);
end;

// baudrate am aktuell geffneten com-port einstellen
function ConfigComPort (baudrate : dword) : boolean;
var dcb : TDCB;
begin
  result := false;
  if GetCommState (tComPortHandle, dcb) then
  begin
    dcb.BaudRate   := baudrate;
    dcb.StopBits   := OneStopBit;
    dcb.ByteSize   := 8;
    dcb.Parity     := NOPARITY;
    dcb.XonChar    := #0;
    dcb.XoffChar   := #0;
    dcb.ErrorChar  := #0;
    dcb.EofChar    := #0;
    dcb.EvtChar    := #0;
    result         := SetCommState (tComPortHandle, dcb);
  end;
end;

// string per com-port ausgeben
// es wird zuerst in einen byte-buffer umkopiert und dann gesendet
function WriteComPort (x : string) : boolean;
var nbw  : DWORD;
    nbwt : DWORD;
    buf  : array [0 .. 65535] of byte;
    c    : integer;
begin
  result := false;
  // nur wenn cm-port geffnet
  if tComPortHandle <> INVALID_HANDLE_VALUE then
  begin
    // zuerst string in buffer umkopieren
    for c := 0 to length (x) - 1 do
      buf [c] := ord (x [c + 1]);
    nbw := length (x);
    // und dann senden ...
    result := WriteFile (tComPortHandle, buf, nbw, nbwt, NIL) and (nbwt = nbw);
   end;
end;

// anzahl der bytes in der emfpangs-warteschlange ermitteln
function BytesInQueue (var x : longint) : boolean;
var err : dword;
    ps  : pointer;
    com_stat        : COMSTAT;
begin
  result := false;
  x := 0;
  // nur wenn com-port geffnet
  if tComPortHandle <> INVALID_HANDLE_VALUE then
  begin
    ps := @com_stat;
    result := ClearCommError (tComPortHandle, err , ps);
    if result then x := com_stat.cbInQue;
  end;
end;

// string aus empfangswarteschlange lesen (wenn denn was da ist)
function ReadComPort (var x : string) : boolean;
var buf  : array [0 .. 65535] of byte;
    c, l : integer; nrd : dword;
begin
  result := false; x := '';
  // nur wenn com-port geffnet
  if tComPortHandle <> INVALID_HANDLE_VALUE then
  begin
    // ist was da ?
    result :=            BytesInQueue (l) and (l > 0);
    // dann buffer - lesen
    result := result and ReadFile (tComPortHandle, buf, l, nrd, NIL) and (l = integer (nrd));
    if result then
    begin
      // und wieder in string umkopieren
      for c := 0 to l - 1 do
        x := x + chr (buf [c]);
      result := true;
    end;
  end;
end;



//////////////////////////////////////////////////////////////////////////////
//
//
//  Kapselung der "losen" Pascal Funktionen in der Klasse TSimple232
//
//
//////////////////////////////////////////////////////////////////////////////

// TSimple232 und Polling Timer erzeugen
constructor TSimple232.Create;
begin
  inherited Create;

  clTimer1 := TTimer.Create (NIL);
  clTimer1.Interval := 20;
  clTimer1.OnTimer := vPoll;
end;

// TSimple232 und Polling Timer zerstren
destructor  TSimple232.Destroy;
begin
  clTimer1.Destroy;
  CloseComPort;
  InitCom;
  inherited;
end;

// Com-Port ffnen und Baudrate einstellen
function  TSimple232.OpenPort (ucComPort : byte; uiBaudRate : dword) : boolean;
begin
  result := false;
  if OpenComPort (ucComPort) then
  begin
    ConfigComPort (uiBaudrate);
    result := true;
  end;
end;

// Com-Port schlieen
procedure TSimple232.ClosePort;
begin
  CloseComPort;
end;

// Com-Port pollen
procedure TSimple232.vPoll (clSender : TObject);
var aucData : string;
begin
  // wenn port offen hole daten ab
  if bComOk and ReadComPort (aucData) then
  begin
    // wenn daten und callback da, dann callback
    if (length (aucData) > 0) and (@pfOnReceiveData <> NIL) then pfOnReceiveData (self, aucData);
  end;
end;

// com-port status holen (geffnet oder nicht)
function TSimple232.F_bComOk : boolean;
begin
  // einfach die handle befragen
  result := tComPortHandle <> INVALID_HANDLE_VALUE;
end;

// kapselung fr com-port senden
procedure TSimple232.vSendData (aucData : string);
begin
  // wenn port offen
  if bComOk then
  begin
    // ...
    WriteComPort (aucdata);
  end;
end;


initialization

  // am anfang immer schn die handle rurcksetzen
  InitCom;

finalization

end.
 