////////////////////////////////////////////////////////////////////////////
//
//  SYNTHTOOLS.PAS
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : April 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit synthtools;

interface


uses classes, sysutils, windows, math,

     fpga_dap;


type  TADSR = class
      protected
       ainc, dinc, rinc, slev : integer;
       glev, clev, olev       : integer;
       estate                 : byte;

      public
        constructor Create;
        destructor  Destroy;

        procedure SetEnv (a, d, s, r : single);
        function  DoEnv (cgate :  single) : integer;
      end;

procedure DAPWriteWTSine  (dap  : TFPGADAP);
procedure DAPWriteWTTri   (dap  : TFPGADAP);


implementation

procedure DAPWriteWTSine (dap  : TFPGADAP);
var x   : integer;
    phi : single;
    y   : smallint;
begin
  dap.WriteMEMStart (DAP_MEM_WMEM);
  for x := 0 to 127 do
  begin
    phi := x / 128;
    y   := round ($7fff * sin (phi * 2 * pi));
    dap.WriteMEMWord (y);
  end;
  dap.WriteMEMEnd;
end;


procedure DAPWriteWTTri (dap  : TFPGADAP);
var x   : integer;
    phi : single;
    y   : word;
begin
  dap.WriteMEMStart (DAP_MEM_WMEM);
  for x := 0 to 127 do
  begin
    phi := x / 127;
    y   := round ($7fff * sin (phi * 2 * pi));
    dap.WriteMEMWord (y);
  end;
  dap.WriteMEMEnd;
end;


constructor TADSR.Create;
begin
  inherited;
  glev    := 0;
  clev    := 0;
  olev    := 0;
  estate  := 0;

  SetEnv (0.02, 0.02, 0.7, 0.02);
end;

destructor  TADSR.Destroy;
begin
  inherited;
end;


procedure TADSR.SetEnv (a, d, s, r : single);
begin
  a := max (0.02, min (5.0, a));
  d := max (0.02, min (5.0, d));
  s := max (0.0 , min (1.0, s));
  r := max (0.02, min (5.0, r));

  ainc := trunc (65535 / (a / 0.02));
  dinc := trunc (65535 / (d / 0.02));
  rinc := trunc (65535 / (r / 0.02));
  slev := trunc (s * 65535);
end;

function TADSR.DoEnv (cgate :  single) : integer;
begin
  case estate of
    0 : begin
          if cgate <> 0 then
          begin
            estate := 1; clev := 0;
          end;
        end;
    1 : begin
          if cgate <> 0 then
            begin
              if (65535 - clev) < ainc then
                begin
                  clev := 65535;
                  estate := 2;
                end
              else
                begin
                  clev := clev + ainc;
                end;
            end
          else
            begin
              estate := 4;
            end;
        end;
    2 : begin
          if cgate <> 0 then
            begin
              if (clev - slev) < dinc then
                begin
                  clev := slev;
                  estate := 3;
                end
              else
                begin
                  clev := clev - dinc;
                end;
            end
          else
            begin
              estate := 4;
            end;
        end;
    3 : begin
          if cgate = 0 then
            begin
              estate := 4;
            end;
        end;
    4 : begin
          if cgate = 0 then
            begin
              if (clev >= rinc) then
                begin
                  clev := clev - rinc;
                end
              else
                begin
                  clev := 0;
                  estate := 0;
                end;
            end
          else
            begin
              estate := 1;
            end;
        end;
  end;
  result := clev;
end;






end.
