////////////////////////////////////////////////////////////////////////////
//
//  DELAY_DEMO.PAS
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit delay_demo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, math, ExtCtrls,

  comselect,
  fpga_com_base, codec_interface,

  fpga_dap,
  coeffs, ComCtrls;



type
  TfrmMain = class(TForm)
    btReset: TButton;
    btMute: TButton;
    tbDelay1: TTrackBar;
    tbDelay2: TTrackBar;
    tbWet: TTrackBar;
    lbDel1: TLabel;
    lbDel2: TLabel;
    lbWet: TLabel;
    lbHdDel1: TLabel;
    lbHdDel2: TLabel;
    lbHdWet: TLabel;
    tmMain: TTimer;
    btMove: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btResetClick(Sender: TObject);
    procedure btMuteClick(Sender: TObject);
    procedure tbDelay1Change(Sender: TObject);
    procedure tmMainTimer(Sender: TObject);
    procedure btMoveClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    fpga    : TFPGACOMBase;
    dap     : TFPGADAP;

    codec1  : TCodecInterface;
    codec2  : TCodecInterface;

    run     : boolean;

    v, vi   : integer;


    del1, del2 : integer;
    wet        : integer;

    procedure InitDAPProgram;
    procedure InitDelay;    
    procedure UpdateDelay;

    procedure SetDelay (num : integer; delay : single);
    procedure SetWet   (w : single);

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
var cp, br : integer;
begin
  fpga  := TFPGACOMBase.Create;

  cp := 1;
  br := fpgacom_baudrate;

  if ComPortSelect (cp, br) then fpga.OpenPort(cp);

  codec1    := TCodecInterface.Create (fpga, 0);
  codec2    := TCodecInterface.Create (fpga, 1);


//  codec1.CodecRegister [0] := $

  dap       := TFPGADAP.Create (fpga);
  dap.Port  := $f1;

  InitDAPProgram;

  del1 := 0;
  del2 := 0;

  tbDelay1.position := 1000 - 375;
  tbDelay2.position := 1000 - 500;
  tbWet.position    := 100  -  25;

  UpdateDelay;

  v := 100;
  vi := 1;

  run := false;

end;



procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  dap.Destroy;
  fpga.destroy;
end;

procedure TfrmMain.InitDAPProgram;
begin
  dap.PMEM_StartProgram;


  dap.PMEM_Store  ( DAP_IN_1 , DAP_OUT_SC_1, $00      );
  dap.PMEM_Store  ( DAP_IN_2 , DAP_OUT_SC_2, $01      );

  dap.PMEM_Delay  ( DAP_IN_1 , DAP_OUT_UC_1, $00      );
  dap.PMEM_Volume ( DAP_IN_UC, DAP_OUT_UC_1, $00      );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_1, $02      );

  dap.PMEM_Delay  ( DAP_IN_2 , DAP_OUT_UC_2, $08      );
  dap.PMEM_Volume ( DAP_IN_UC, DAP_OUT_UC_2, $00      );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_2, $03      );


  dap.PMEM_ClrSum (            DAP_OUT_SC_1           );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $00 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $02 );
  dap.PMEM_Nop    ( DAP_IN_SC, DAP_OUT_SC_1           );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_1, $04      );



  dap.PMEM_ClrSum (            DAP_OUT_SC_2           );
  dap.PMEM_AddMem (            DAP_OUT_SC_2,      $01 );
  dap.PMEM_AddMem (            DAP_OUT_SC_2,      $03 );
  dap.PMEM_Nop    ( DAP_IN_SC, DAP_OUT_SC_2           );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_2, $05      );



  dap.PMEM_Load   (            DAP_OUT_SC_1,      $04 );
  dap.PMEM_Load   (            DAP_OUT_SC_2,      $05 );

  dap.PMEM_EndProgram;

  InitDelay;
end;



procedure TfrmMain.InitDelay;
begin
  dap.WriteMEMStart ($6000);


  dap.WriteMEMLong  ($00000000);
  dap.WriteMEMLong  ($00000000);
  dap.WriteMEMLong  ($0000ffff);
  dap.WriteMEMLong  (del1);

  dap.WriteMEMEnd;


  dap.WriteMEMStart ($6020);

  dap.WriteMEMLong  ($00020000);
  dap.WriteMEMLong  ($00000000);
  dap.WriteMEMLong  ($0000ffff);
  dap.WriteMEMLong  (del2);


  dap.WriteMEMEnd;

  dap.CMEM_Longint($00, wet);
end;

procedure TfrmMain.UpdateDelay;
begin
  dap.WriteMEMStart ($600c);

  dap.WriteMEMLong  (del1);
  dap.WriteMEMEnd;

  dap.WriteMEMStart ($602c);
  dap.WriteMEMLong  (del2);

  dap.WriteMEMEnd;

  dap.CMEM_Longint($00, wet);
end;


procedure TfrmMain.SetDelay (num : integer; delay : single);
begin
  num   := min (2, max (1, num));
  delay := min (1.365, max (0, delay)) * 48000;
  if num = 1 then del1 :=  trunc (delay);
  if num = 2 then del2 :=  trunc (delay);
end;

procedure TfrmMain.SetWet (w : single);
begin
  w := min (1, max (0, w)) * $01000000;
  wet := trunc (w);
end;





procedure TfrmMain.btMoveClick(Sender: TObject);
begin
  if btMove.Caption = 'Move It !'  then btMove.Caption := 'Stop'
                                   else btMove.Caption := 'Move It !';
  run := btMove.Caption = 'Stop';
  tbdelay1.Enabled := not run;
  tbdelay2.Enabled := not run;
  lbHdDel1.Enabled := not run;
  lbHdDel2.Enabled := not run;
  lbDel1.Enabled := not run;
  lbDel2.Enabled := not run;

end;

procedure TfrmMain.btMuteClick(Sender: TObject);
begin
  if btMute.Caption = 'Mute'  then btMute.Caption := 'Unmute'
                              else btMute.Caption := 'Mute';
  dap.Mute(btMute.Caption = 'Unmute');
end;

procedure TfrmMain.btResetClick(Sender: TObject);
begin
  InitDAPProgram;
end;




procedure TfrmMain.tbDelay1Change(Sender: TObject);
begin
  if sender = tbDelay1 then setdelay (1, 1 - (tbdelay1.Position / 1000));
  if sender = tbDelay2 then setdelay (2, 1 - (tbdelay2.Position / 1000));
  if sender = tbWet    then setwet   (   1 - (tbwet.position / 100));

  lbDel1.Caption := format ('%d ms', [(1000 - tbdelay1.Position)]);
  lbDel2.Caption := format ('%d ms', [(1000 - tbdelay2.Position)]);
  lbWet .Caption := format ('%d %%', [(100   - tbwet   .Position)]);


  UpdateDelay;
end;

procedure TfrmMain.tmMainTimer(Sender: TObject);
begin
  if run then
  begin
    if v > 240 then vi := -vi;
    if v <  10 then vi := -vi;

    v := v + vi;

    del1 := v;
    del2 := 255 - v;

    updatedelay;
  end;
end;

end.
