////////////////////////////////////////////////////////////////////////////
//
//  EQ_DEMO.PAS
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit eq_demo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls,

  comselect,
  fpga_com_base, codec_interface,

  fpga_dap,
  coeffs;

type
  TfrmMain = class(TForm)
    btReset: TButton;
    btMute: TButton;
    btMove: TButton;
    btFlat: TButton;
    Timer1: TTimer;
    tbEQ1: TTrackBar;
    tbEQ2: TTrackBar;
    tbEQ3: TTrackBar;
    lbEQ1: TLabel;
    lbEQ2: TLabel;
    lbEQ3: TLabel;
    lbHd1: TLabel;
    lbHd2: TLabel;
    lbHd3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btMoveClick(Sender: TObject);
    procedure btResetClick(Sender: TObject);
    procedure btMuteClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure tbEQ1Change(Sender: TObject);
    procedure btFlatClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    fpga    : TFPGACOMBase;
    dap     : TFPGADAP;

    codec1  : TCodecInterface;
    codec2  : TCodecInterface;    

    run     : boolean;


    v1, vi1 : single;
    v2, vi2 : single;
    v3, vi3 : single;

    eq_values : array [0 .. 2] of single;


    procedure InitDAPProgram;

    procedure UpdateEq;

  end;

var
  frmMain: TfrmMain;



const samplerate : integer = 48000;

implementation

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
var cp, br : integer;
begin
  fpga  := TFPGACOMBase.Create;

  cp := 1;
  br := fpgacom_baudrate;

  if ComPortSelect (cp, br) then fpga.OpenPort(cp);

  codec1    := TCodecInterface.Create (fpga, 0);
  codec2    := TCodecInterface.Create (fpga, 1);

  dap       := TFPGADAP.Create (fpga);
  dap.Port  := $f1;

  InitDAPProgram;

  run  := false;

  v1 :=  0; vi1 := 0.1;
  v2 :=  3; vi2 := 0.15;
  v3 := -3; vi3 := 0.075;

  eq_values [0] := 0.0;
  eq_values [1] := 0.0;
  eq_values [2] := 0.0;

  UpdateEQ;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  dap.Destroy;
  fpga.destroy;
end;

procedure TfrmMain.InitDAPProgram;
begin
  dap.PMEM_StartProgram;

  dap.PMEM_Biquad ( DAP_IN_1 , DAP_OUT_SC_1, $00, $00 );
  dap.PMEM_Biquad ( DAP_IN_SC, DAP_OUT_SC_1, $08, $08 );
  dap.PMEM_Biquad ( DAP_IN_SC, DAP_OUT_SC_1, $10, $10 );

  dap.PMEM_Biquad ( DAP_IN_2 , DAP_OUT_SC_2, $40, $00 );
  dap.PMEM_Biquad ( DAP_IN_SC, DAP_OUT_SC_2, $48, $08 );
  dap.PMEM_Biquad ( DAP_IN_SC, DAP_OUT_SC_2, $50, $10 );

  dap.PMEM_EndProgram;
end;


procedure TfrmMain.UpdateEq;
var c : TDAPBiquad;
begin
  calccoeffpk (300, 1.41, eq_values [0], c);
  dap.CMEM_Biquad ($00, c);

  calccoeffpk (1000, 1.41, eq_values [1], c);
  dap.CMEM_Biquad ($08, c);

  calccoeffpk (3000, 1.41, eq_values [2], c);
  dap.CMEM_Biquad ($10, c);

end;



procedure TfrmMain.btResetClick(Sender: TObject);
begin
  InitDAPProgram;
  UpdateEq;
end;

procedure TfrmMain.btMuteClick(Sender: TObject);
begin
  if btMute.Caption = 'Mute'  then btMute.Caption := 'Unmute'
                              else btMute.Caption := 'Mute';
  dap.Mute(btMute.Caption = 'Unmute');
end;

procedure TfrmMain.btFlatClick(Sender: TObject);
begin
  tbeq1.Position := 0;
  tbeq2.Position := 0;
  tbeq3.Position := 0;
end;

procedure TfrmMain.btMoveClick(Sender: TObject);
begin
  if btMove.Caption = 'Move It !'  then btMove.Caption := 'Stop'
                                   else btMove.Caption := 'Move It !';
  run := btMove.Caption = 'Stop';
end;



procedure TfrmMain.Timer1Timer(Sender: TObject);
begin
  if run then
  begin
    if v1 > 11  then vi1 := -vi1;
    if v1 < -11 then vi1 := -vi1;

    v1 := v1 + vi1;
    tbeq1.position := trunc (-v1 * 10);

    if v2 > 11  then vi2 := -vi2;
    if v2 < -11 then vi2 := -vi2;

    v2 := v2 + vi2;
    tbeq2.position := trunc (-v2 * 10);

    if v3 > 11  then vi3 := -vi3;
    if v3 < -11 then vi3 := -vi3;

    v3 := v3 + vi3;
    tbeq3.position := trunc (-v3 * 10);
  end;
end;



procedure TfrmMain.tbEQ1Change(Sender: TObject);
begin
  if sender = tbeq1 then eq_values [0] := -tbeq1.position / 10;
  if sender = tbeq2 then eq_values [1] := -tbeq2.position / 10;
  if sender = tbeq3 then eq_values [2] := -tbeq3.position / 10;

  UpdateEq;

  lbeq1.caption := format ('%.1fdB', [eq_values [0]]);
  lbeq2.caption := format ('%.1fdB', [eq_values [1]]);
  lbeq3.caption := format ('%.1fdB', [eq_values [2]]);

end;



end.
